@@ -29,13 +29,15 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <nautilus-burn-drive.h>
 #include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <profiles/gnome-media-profiles.h>
 #include <gst/gst.h>
 
+#include "bacon-message-connection.h"
 #include "sj-about.h"
-#include "sj-genres.h"
 #include "sj-metadata.h"
 #include "sj-metadata-musicbrainz.h"
 #include "sj-extractor.h"
@@ -43,6 +45,7 @@
 #include "sj-error.h"
 #include "sj-util.h"
 #include "sj-prefs.h"
+#include "sj-play.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 
@@ -61,14 +64,15 @@ SjExtractor *extractor;
 GConfClient *gconf_client;
 
 GtkWidget *main_window;
-static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_combo;
-static GtkWidget *track_listview, *extract_button;
+static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
+static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
-static GtkWidget *extract_menuitem, *select_all_menuitem, *deselect_all_menuitem;
+static GtkWidget *extract_menuitem, *play_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 GtkListStore *track_store;
+static BaconMessageConnection *connection;
 
-const char *base_path, *path_pattern, *file_pattern;
-const char *device = NULL;
+const char *base_uri, *path_pattern, *file_pattern;
+NautilusBurnDrive *drive = NULL;
 gboolean strip_chars;
 gboolean eject_finished;
 gboolean extracting = FALSE;
@@ -78,9 +82,24 @@ static gint total_no_of_tracks;
 static gint no_of_tracks_selected;
 static AlbumDetails *current_album;
 
-int autostart = FALSE;
+gboolean autostart = FALSE, autoplay = FALSE;
 
 #define DEFAULT_PARANOIA 4
+#define RAISE_WINDOW "raise-window"
+#define SOURCE_GLADE "../data/sound-juicer.glade"
+#define INSTALLED_GLADE DATADIR"/sound-juicer/sound-juicer.glade"
+
+void
+sj_main_set_title (const char* detail)
+{
+  if (detail == NULL) {
+    gtk_window_set_title (GTK_WINDOW (main_window), _("Sound Juicer"));
+  } else {
+    char *s = g_strdup_printf ("%s - %s", detail, _("Sound Juicer"));
+    gtk_window_set_title (GTK_WINDOW (main_window), s);
+    g_free (s);
+  }
+}
 
 static void error_on_start (GError *error)
 {
@@ -120,7 +139,10 @@ void on_destroy_signal (GtkMenuItem *item, gpointer user_data)
  */
 void on_eject_activate (GtkMenuItem *item, gpointer user_data)
 {
-  eject_cdrom (device, GTK_WINDOW (main_window));
+  /* first make sure we're not playing */
+  stop_playback ();
+
+  nautilus_burn_drive_eject (drive);
 }
 
 gboolean poll_tray_opened (gpointer data)
@@ -130,7 +152,7 @@ gboolean poll_tray_opened (gpointer data)
   if (extracting == TRUE)
     return TRUE;
 
-  new_status = tray_is_opened (device);
+  new_status = nautilus_burn_drive_door_is_open (drive);
   if (new_status != tray_opened && new_status == FALSE) {
     reread_cd (TRUE);
   } else if (new_status != tray_opened && new_status == TRUE) {
@@ -219,6 +241,30 @@ static void duration_cell_data_cb (GtkTreeViewColumn *tree_column,
   g_free (string);
 }
 
+extern gint current_track;
+static void title_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
+                                GtkCellRenderer *cell,
+                                GtkTreeModel *tree_model,
+                                GtkTreeIter *iter,
+                                gpointer data)
+{
+  TrackState state;
+  gtk_tree_model_get (tree_model, iter, COLUMN_STATE, &state, -1);
+  switch (state) {
+  case STATE_IDLE:
+    g_object_set (G_OBJECT (cell), "stock-id", "", NULL);
+    break;
+  case STATE_PLAYING:
+    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_PLAY, NULL);
+    break;
+  case STATE_EXTRACTING:
+    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_RECORD, NULL);
+    break;
+  default:
+    g_warning("Unhandled track state %d\n", state);
+  }
+}
+
 /**
  * Utility function to update the UI for a given Album
  */
@@ -233,11 +279,13 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_list_store_clear (track_store);
     gtk_entry_set_text (GTK_ENTRY (title_entry), "");
     gtk_entry_set_text (GTK_ENTRY (artist_entry), "");
+    gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
     gtk_label_set_text (GTK_LABEL (duration_label), "");
-    gtk_combo_box_set_active (GTK_COMBO_BOX (genre_combo), -1);
     gtk_widget_set_sensitive (title_entry, FALSE);
     gtk_widget_set_sensitive (artist_entry, FALSE);
-    gtk_widget_set_sensitive (genre_combo, FALSE);
+    gtk_widget_set_sensitive (genre_entry, FALSE);
+    gtk_widget_set_sensitive (play_button, FALSE);
+    gtk_widget_set_sensitive (play_menuitem, FALSE);
     gtk_widget_set_sensitive (extract_button, FALSE);
     gtk_widget_set_sensitive (extract_menuitem, FALSE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
@@ -252,10 +300,11 @@ static void update_ui_for_album (AlbumDetails *album)
     g_signal_handlers_unblock_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_unblock_by_func (artist_entry, on_artist_edit_changed, NULL);
 
-    gtk_combo_box_set_active (GTK_COMBO_BOX (genre_combo), -1);
     gtk_widget_set_sensitive (title_entry, TRUE);
     gtk_widget_set_sensitive (artist_entry, TRUE);
-    gtk_widget_set_sensitive (genre_combo, TRUE);
+    gtk_widget_set_sensitive (genre_entry, TRUE);
+    gtk_widget_set_sensitive (play_button, TRUE);
+    gtk_widget_set_sensitive (play_menuitem, TRUE);
     gtk_widget_set_sensitive (extract_button, TRUE);
     gtk_widget_set_sensitive (extract_menuitem, TRUE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
@@ -267,6 +316,7 @@ static void update_ui_for_album (AlbumDetails *album)
       album_duration += track->duration;
       gtk_list_store_append (track_store, &iter);
       gtk_list_store_set (track_store, &iter,
+                          COLUMN_STATE, STATE_IDLE,
                           COLUMN_EXTRACT, TRUE,
                           COLUMN_NUMBER, track->number,
                           COLUMN_TITLE, track->title,
@@ -276,7 +326,7 @@ static void update_ui_for_album (AlbumDetails *album)
                           -1);
      total_no_of_tracks++; 
     }
-no_of_tracks_selected=total_no_of_tracks;
+    no_of_tracks_selected=total_no_of_tracks;
 
     /* Some albums don't have track durations :( */
     if (album_duration) {
@@ -373,15 +423,15 @@ AlbumDetails* multiple_album_dialog(GList *albums)
 /**
  * The GConf key for the base path changed
  */
-void basepath_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
-  g_assert (strcmp (entry->key, GCONF_BASEPATH) == 0);
+  g_assert (strcmp (entry->key, GCONF_BASEURI) == 0);
   if (entry->value == NULL) {
-    base_path = g_strdup (g_get_home_dir ());
+    base_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
   } else {
-    base_path = gconf_value_get_string (entry->value);
+    base_uri = gconf_value_get_string (entry->value);
   }
-  /* TODO: sanity check the path somewhat */
+  /* TODO: sanity check the URI somewhat */
 }
 
 /**
@@ -468,14 +518,15 @@ metadata_cb (SjMetadata *m, GList *albums, GError *error)
   
   if (error && !(error->code == SJ_ERROR_CD_NO_MEDIA)) {
     GtkWidget *dialog;
+    
     dialog = gtk_message_dialog_new (realized ? GTK_WINDOW (main_window) : NULL, 0,
                                      GTK_MESSAGE_ERROR,
                                      GTK_BUTTONS_CLOSE,
-				     "<b>%s</b>\n\n%s\n%s: %s",
-				      _("Could not read the CD"),
-				      _("Sound Juicer could not read the track listing on this CD."),
-				      _("Reason"),
-				      error->message);
+                                     "<b>%s</b>\n\n%s\n%s: %s",
+                                     _("Could not read the CD"),
+                                     _("Sound Juicer could not read the track listing on this CD."),
+                                     _("Reason"),
+                                     error->message);
     gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -509,7 +560,21 @@ metadata_cb (SjMetadata *m, GList *albums, GError *error)
   
   if (autostart) {
     g_signal_emit_by_name (extract_button, "activate", NULL);
+    autostart = FALSE;
   }
+  if (autoplay) {
+    g_signal_emit_by_name (play_button, "activate", NULL);
+    autoplay = FALSE;
+  }
+}
+
+static gboolean
+is_audio_cd (NautilusBurnDrive *drive)
+{
+  gboolean audio;
+  if (drive == NULL) return FALSE;
+  nautilus_burn_drive_get_media_type_full (drive, NULL, NULL, NULL, &audio);
+  return audio;
 }
 
 /**
@@ -534,7 +599,7 @@ static void reread_cd (gboolean ignore_no_media)
   /* Set statusbar message */
   gtk_statusbar_push(GTK_STATUSBAR(status_bar), 0, _("Retrieving track listing...please wait."));
 
-  if (!is_audio_cd (device)) {
+  if (!is_audio_cd (drive)) {
     update_ui_for_album (NULL);
     gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
     if (realized)
@@ -573,8 +638,9 @@ static void reread_cd (gboolean ignore_no_media)
  */
 void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
-  g_assert (strcmp (entry->key, GCONF_DEVICE) == 0);
+  const char *device;
   gboolean ignore_no_media = GPOINTER_TO_INT (user_data);
+  g_assert (strcmp (entry->key, GCONF_DEVICE) == 0);
 
   if (entry->value == NULL
 		  || !cd_drive_exists (gconf_value_get_string (entry->value))) {
@@ -595,6 +661,7 @@ void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
       exit (1);
 #endif
     }
+    drive = nautilus_burn_drive_new_from_path (device);
   } else {
     device = gconf_value_get_string (entry->value);
     if (access (device, R_OK) != 0) {
@@ -615,13 +682,15 @@ void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
       /* Set a null device */
-      device = NULL;
+      drive = NULL;
+    } else {
+      drive = nautilus_burn_drive_new_from_path (device);
     }
   }
   sj_metadata_set_cdrom (metadata, device);
   sj_extractor_set_device (extractor, device);
 
-  tray_opened = tray_is_opened (device);
+  tray_opened = nautilus_burn_drive_door_is_open (drive);
   if (tray_opened == FALSE) {
     reread_cd (ignore_no_media);
   }
@@ -845,20 +914,12 @@ void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(track_store), &iter));
 }
 
-static gboolean genre_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {
-  TrackDetails *track;
-  gtk_tree_model_get (model, iter, COLUMN_DETAILS, &track, -1);
-  track->genre = GPOINTER_TO_INT(data);
-  return FALSE;
-}
-
-void on_genre_combo_changed(GtkComboBox *combo, gpointer user_data) {
-  GtkTreeIter iter;
-  if (gtk_combo_box_get_active_iter (combo, &iter)) {
-    int num;
-    gtk_tree_model_get (gtk_combo_box_get_model (combo), &iter, 0, &num, -1);
-    gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), (GtkTreeModelForeachFunc)genre_foreach, GINT_TO_POINTER(num));
+void on_genre_edit_changed(GtkEditable *widget, gpointer user_data) {
+  g_return_if_fail (current_album != NULL);
+  if (current_album->genre) {
+    g_free (current_album->genre);
   }
+  current_album->genre = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
 }
 
 void on_contents_activate(GtkWidget *button, gpointer user_data) {
@@ -883,50 +944,77 @@ void on_contents_activate(GtkWidget *button, gpointer user_data) {
   }
 }
 
-static int genre_cmp (gconstpointer a, gconstpointer b)
-{
-  GenreMap *p1, *p2;
-  p1 = (GenreMap *)a;
-  p2 = (GenreMap *)b;
-  return strcmp (sj_genre_name (p1->canonical), sj_genre_name (p2->canonical));
-}
 
-static GtkTreeModel* populate_genre_list(void) {
+static const char* genres[] = {
+  "Ambient",
+  "Blues",
+  "Classical",
+  "Country",
+  "Dance",
+  "Electronica",
+  "Folk",
+  "Jazz",
+  "Latin",
+  "Pop",
+  "Rap",
+  "Reggae",
+  "Rock",
+  "Soul",
+  "Spoken Word",
+  NULL
+};
+
+static GtkTreeModel* create_genre_list(void) {
   GtkListStore *store;
-  GenreMap *p;
-  GList *genres;
-  GList *g;
+  const char **g = genres;
 
-  store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
-  p = genremap;
-  
-  genres = NULL;
-  while (p->genre != LAST_GENRE) {
-    if (p->genre == p->canonical) {
-      genres = g_list_insert_sorted (genres, (gpointer)p, genre_cmp);
-    }
-    ++p;
-  }
+  store = gtk_list_store_new (1, G_TYPE_STRING);
 
-  g = genres;
-  while (g != NULL) {
+  while (*g != NULL) {
     GtkTreeIter iter;
-    p = (GenreMap *)g->data;
     gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, p->canonical, 1, sj_genre_name (p->canonical), -1);
-    g = g->next;
+    gtk_list_store_set (store, &iter, 0, *g++, -1);
   }
-  g_list_free (genres);
 
   return GTK_TREE_MODEL (store);
 }
 
+static void
+upgrade_gconf (void)
+{
+  char *s;
+  s = gconf_client_get_string (gconf_client, GCONF_BASEURI, NULL);
+  if (s != NULL) {
+    g_free (s);
+  } else {
+    char *uri;
+    s = gconf_client_get_string (gconf_client, GCONF_BASEPATH, NULL);
+    if (s == NULL)
+      return;
+    uri = gnome_vfs_get_uri_from_local_path (s);
+    g_free (s);
+    gconf_client_set_string (gconf_client, GCONF_BASEURI, uri, NULL);
+    g_free (uri);
+  }
+}
+
+static void
+on_message_received (const char *message, gpointer user_data)
+{
+  if (message == NULL)
+    return;
+  if (strcmp (RAISE_WINDOW, message) == 0) {
+    gtk_window_present (GTK_WINDOW (main_window));
+  }
+}
+
 int main (int argc, char **argv)
 {
   GError *error = NULL;
   struct poptOption options[] = {
     { NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL },
     { "auto-start", 'a', POPT_ARG_NONE, &autostart, 0, "Start extracting immediately", NULL},
+    { "play", 'p', POPT_ARG_NONE, &autoplay, 0, "Start playing immediately", NULL},
     POPT_TABLEEND
   };
 
@@ -943,6 +1031,15 @@ int main (int argc, char **argv)
                       NULL);
   g_set_application_name (_("Sound Juicer"));
 
+  connection = bacon_message_connection_new ("sound-juicer");
+  if (bacon_message_connection_get_is_server (connection) == FALSE) {
+	  bacon_message_connection_send (connection, RAISE_WINDOW);
+	  bacon_message_connection_free (connection);
+	  exit (0);
+  } else {
+    bacon_message_connection_set_callback (connection, on_message_received, NULL);
+  }
+
   metadata = SJ_METADATA (sj_metadata_musicbrainz_new ());
   error = sj_metadata_get_new_error (metadata);
   if (error) {
@@ -957,10 +1054,12 @@ int main (int argc, char **argv)
     exit (1);
   }
 
+  upgrade_gconf ();
+
   gconf_client_add_dir (gconf_client, GCONF_ROOT, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
   gconf_client_notify_add (gconf_client, GCONF_DEVICE, device_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_EJECT, eject_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (gconf_client, GCONF_BASEPATH, basepath_changed_cb, NULL, NULL, NULL);
+  gconf_client_notify_add (gconf_client, GCONF_BASEURI, baseuri_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_STRIP, strip_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_AUDIO_PROFILE, profile_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_PARANOIA, paranoia_changed_cb, NULL, NULL, NULL);
@@ -975,8 +1074,17 @@ int main (int argc, char **argv)
   gnome_media_profiles_init (gconf_client);
 
   glade_init ();
-  glade = glade_xml_new (DATADIR"/sound-juicer/sound-juicer.glade", NULL, NULL);
-  g_assert (glade != NULL);
+  if (g_file_test (SOURCE_GLADE, G_FILE_TEST_EXISTS) != FALSE) {
+    glade = glade_xml_new (SOURCE_GLADE, NULL, NULL);
+  } else {
+    glade = glade_xml_new (INSTALLED_GLADE, NULL, NULL);
+  }
+  if (glade == NULL) {
+    g_error_new (0, 0, _("The interface file for Sound Juicer could not be read."));
+    error_on_start (error);
+    g_error_free (error);
+    exit (1);
+  }
   glade_xml_signal_autoconnect (glade);
 
   main_window = glade_xml_get_widget (glade, "main_window");
@@ -984,80 +1092,86 @@ int main (int argc, char **argv)
 
   select_all_menuitem = glade_xml_get_widget (glade, "select_all");
   deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
-  extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
   title_entry = glade_xml_get_widget (glade, "title_entry");
   artist_entry = glade_xml_get_widget (glade, "artist_entry");
   duration_label = glade_xml_get_widget (glade, "duration_label");
-  genre_combo = glade_xml_get_widget (glade, "genre_combo");
+  genre_entry = glade_xml_get_widget (glade, "genre_entry");
   track_listview = glade_xml_get_widget (glade, "track_listview");
   extract_button = glade_xml_get_widget (glade, "extract_button");
+  extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
+  play_button = glade_xml_get_widget (glade, "play_button");
+  play_menuitem = glade_xml_get_widget (glade, "play_menuitem");
   status_bar = glade_xml_get_widget (glade, "status_bar");
 
-  gtk_combo_box_set_model (GTK_COMBO_BOX (genre_combo), populate_genre_list ());
   {
-    GtkCellRenderer *renderer;
-    renderer = gtk_cell_renderer_text_new ();
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT(genre_combo), renderer, TRUE);
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(genre_combo), renderer, "text", 1);
+    GtkEntryCompletion *completion;
+    completion = gtk_entry_completion_new ();
+    gtk_entry_completion_set_model (completion, create_genre_list ());
+    gtk_entry_completion_set_text_column (completion, 0);
+    gtk_entry_completion_set_inline_completion (completion, TRUE);
+    gtk_entry_set_completion (GTK_ENTRY (genre_entry), completion);
   }
 
-  track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
+  track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
   gtk_tree_view_set_model (GTK_TREE_VIEW (track_listview), GTK_TREE_MODEL (track_store));
   {
     GtkTreeViewColumn *column;
-    GtkCellRenderer *text_renderer, *toggle_renderer;
+    GtkCellRenderer *renderer;
     
-    toggle_renderer = gtk_cell_renderer_toggle_new ();
-    g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_extract_toggled), NULL);
+    renderer = gtk_cell_renderer_toggle_new ();
+    g_signal_connect (renderer, "toggled", G_CALLBACK (on_extract_toggled), NULL);
     column = gtk_tree_view_column_new_with_attributes (_("Extract"),
-                                                       toggle_renderer,
+                                                       renderer,
                                                        "active", COLUMN_EXTRACT,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    text_renderer = gtk_cell_renderer_text_new ();
+    renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Track"),
-                                                       text_renderer,
+                                                       renderer,
                                                        "text", COLUMN_NUMBER,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    /* TODO: Do I need to create these every time or will a single one do? */
-    text_renderer = gtk_cell_renderer_text_new ();
-    g_signal_connect (text_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_TITLE));
-    g_object_set (G_OBJECT (text_renderer), "editable", TRUE, NULL);
-    column = gtk_tree_view_column_new_with_attributes (_("Title"),
-                                                       text_renderer,
-                                                       "text", COLUMN_TITLE,
-                                                       NULL);
+    column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_title (column, _("Title"));
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_expand (column, TRUE);
+    renderer = gtk_cell_renderer_pixbuf_new ();
+    gtk_tree_view_column_pack_start (column, renderer, FALSE);
+    gtk_tree_view_column_set_cell_data_func (column, renderer, title_cell_icon_data_cb, NULL, NULL);
+
+    renderer = gtk_cell_renderer_text_new ();
+    g_signal_connect (renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_TITLE));
+    g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
+    gtk_tree_view_column_pack_start (column, renderer, TRUE);
+    gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_TITLE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    text_renderer = gtk_cell_renderer_text_new ();
+    renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Artist"),
-                                                       text_renderer,
+                                                       renderer,
                                                        "text", COLUMN_ARTIST,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_expand (column, TRUE);
-    g_signal_connect (text_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
-    g_object_set (G_OBJECT (text_renderer), "editable", TRUE, NULL);
+    g_signal_connect (renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
+    g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
     
-    text_renderer = gtk_cell_renderer_text_new ();
+    renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Duration"),
-                                                       text_renderer,
+                                                       renderer,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
-    gtk_tree_view_column_set_cell_data_func (column, text_renderer, duration_cell_data_cb, NULL, NULL);
+    gtk_tree_view_column_set_cell_data_func (column, renderer, duration_cell_data_cb, NULL, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
   }
 
   update_ui_for_album (NULL);
-  gtk_widget_show_all (main_window);
+  gtk_widget_show (main_window);
 
   extractor = SJ_EXTRACTOR (sj_extractor_new());
   error = sj_extractor_get_new_error (extractor);
@@ -1067,7 +1181,8 @@ int main (int argc, char **argv)
   }
 
   http_proxy_setup (gconf_client);
-  basepath_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEPATH, NULL, TRUE, NULL), NULL);
+  /* TODO: port port basepath to baseuri */
+  baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
   path_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PATH_PATTERN, NULL, TRUE, NULL), NULL);
   file_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FILE_PATTERN, NULL, TRUE, NULL), NULL);
   profile_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_PROFILE, NULL, TRUE, NULL), NULL);
@@ -1082,7 +1197,7 @@ int main (int argc, char **argv)
   }
 
   /* Poke the CD drive every now and then */
-  tray_opened = tray_is_opened (device);
+  tray_opened = nautilus_burn_drive_door_is_open (drive);
   poll_id = g_timeout_add (2000, poll_tray_opened, NULL);
 
   gtk_main ();
